// file: main.C for wfAbc, a ABC simulation program designed to estimate s and h for L11 in a similar manner to what we have for L14, L1 L2 and L3. Here we only L11-F4 and the L11-F4 parents.


// Time-stamp: <Wednesday, 19 July 2017, 14:06 MDT -- zgompert>

#include <cmath>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <gsl/gsl_matrix.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_rng.h>
#include <time.h>
#include <getopt.h>
#include "beetle.H"

using namespace std;

gsl_rng * r;  /* global state variable for random number generator */

/* ----------------- */
/* beginning of main */
/* ----------------- */

int main(int argc, char *argv[]) {

  int ch = 0;
  time_t start = time(NULL);
  time_t end;
  int rng_seed = 0;
  int rep;
  
  string pfile = "undefined";


  dataset data;
  parameters parm;

  data.reps = 1000; // number of abc sims
  data.snp = 0; // SNP number to analyze
  data.esth = 0; // estimate het effect
  
  parm.sigma = 0.1;
  
  // get command line arguments
  if (argc < 2) {
    usage(argv[0]);
  }
  
  while ((ch = getopt(argc, argv, "i:l:n:v:h:")) != -1){
    switch(ch){
    case 'i':
      pfile = optarg;
      break;
    case 'l':
      data.snp = atoi(optarg);
      break;
    case 'n':
      data.reps = atoi(optarg);
      break;
    case 'v':
      parm.sigma = atof(optarg);
      break;
    case 'h':
      data.esth = atoi(optarg);
      break;
    case '?': 
    default:
      usage(argv[0]);
    }
  }
  
  // set up gsl random number generation 
  gsl_rng_env_setup();
  r = gsl_rng_alloc (gsl_rng_default);
  srand(time(NULL));
  rng_seed = rand();
  gsl_rng_set(r, rng_seed); /* seed gsl_rng with output of rand, which
                               was seeded with result of time(NULL) */

  // allocate memory
  data.varNe = gsl_vector_calloc(NNE);
  data.t = gsl_vector_calloc(NNE);

  // read infiles, record genotype likelihoods and data dimensions
  cerr << "Reading data: " << pfile << ", SNP no. " << 
    data.snp << endl;
  getdata(pfile, &data);

  // simulations for abc sims
  for (rep=0; rep < data.reps; rep++)
    wfsim(&data, &parm);

  // prints run time
  end = time(NULL);
  cerr << "Runtime: " << (end-start)/3600 << " hr " << (end-start)%3600/60 << " min ";
  cerr << (end-start)%60 << " sec" << endl;
  return 0;
}
